%% Generation of test data
%   Author: tckb
%   Last modification date: 10/08/2012
%
% [out save_dir]  = generateTestData(data_loc,n,len,[save,r])
%
% DESC:
%    Generates test sample(s) 'out' by randomly choosing test data located
%    at 'data_loc'
%
%  eg., for
%              data_loc = [ 1,2,3,4,5 ]
%       generateTestData(data_loc,1,4) might return out = [ 5,7,1,7 ]
%
%
% INPUT:
%   data_loc    :=  /path/to/data from which sample(s) has to be generated.
%   n           := #test samples to generate
%   len         := #elements in each test sample.
%   save        :=  boolean,
%                           true - save the generated sample(s) to disk.
%                           By default, saved at  data_loc/samples/<out(n).name>.wav
%                           false - do not save (default)
%   r           :=  0 - no repetetion [1]
%                   k - #repetetions in each test sample (not implemented)
%
% OUTPUT:
%   out(n)      := generated test sample(s)
%
%       .srate  := sampling rate of the generated sample [2]
%       .data   := the data
%       .name   := name of the generated sample
%
% NOTE:
%  [1] Repetetions is allowed by default; use r=0 to turn off repetetion.
%  [2] In-consistencies in sampling rate amoungst the test data would yield
%      unpredictable sampling rate (although with-in test data) of generated sample.


function  [out,save_dir]  = generateTestData(data_loc,n,len,save,r)


if nargin <3 || nargin > 5
    error('Too many or too less parameters. Type "help generateTestData" for usage.');
else
    if nargin == 3
        save=false;
        r=1;
    end
    if n <= 0
        disp(' WARNING: n<=0 No test samples are generated! ');
        out.srate=0;
        out.name='null';
        out.data=(0);
    else
        
        tid=tic();
        % get the list of .wav files in the data_loc
        
        testfiles = dir(fullfile(data_loc,'/*.wav'));
        N = size(testfiles,1);
        disp(['#data samples found: ',num2str(N)]);
        
        
        % number of elements to choose
        R = len;
        if r == 0
            comb = combinator(N,R,'c'); % Combining without repetetions
            % n_comb := ...
        else
            comb = combinator(N,R,'c','r'); % combine 'R' elements from a list of 'N' elements
            % its a problem of combination
            % with repetetions.
            % n_comb := (N+R-1)! / R! * (N-1)
        end
        
        
        n_comb = size(comb,1);
        
        
        % This is the limiting case!
        if n > n_comb
            disp([' WARNING: Limit reached! Only ',num2str(n_comb),' samples can be generated']);
            n=n_comb;
        end
        
        
        
        
        % now randomly pick 'n' samples from the comb
        rix = randi(n_comb,n,1);
        % rix contains the indices which are randomly choosen from the combination list
        
        
        r_list =  zeros(n,R);
        
        % now pick the elements from the combination list
        for j=1:size(rix,1)
            r_list(j,:)= comb(rix(j,1),:);
        end
        
        
        
        % now extract the files at from each randomly picked element
        disp(['Generating ',num2str(n),' sample(s) of length ', num2str(len)]);
        
        for k=1:size(r_list,1)
            
            
            outData =[];
            srate=0;
            name=[];
            
            for l=1:size(r_list,2)
                
                file=testfiles(r_list(k,l));
                [data, fs] = wavread([data_loc,'/',file.name]);
                [fp,fname,fext] = fileparts([data_loc,'/',file.name]);
                
                outData = vertcat(outData ,data); % append each file
                srate = fs;
                name= [name,'_',fname];
                
            end
            
            out(k).data = outData;
            out(k).srate = srate;
            out(k).name = name ;
            
        end
        disp('...done');
        
        
        
        % if flag is enabled, save the files to disk
        
        if save
            
            save_dir = [data_loc,'/gen_samples']; % give reference to directory where the files are stored
            disp(['Saving ',num2str(n),' sample(s) @',save_dir]);
            
            % create out directory
            [flag,m,mid]=mkdir(save_dir);
            if flag == 0
                error(' ERROR: Saving samples failed; Can not create sample directory ');
            else
                
                
                for k=1:size(r_list,1)
                    wavwrite(out(k).data,out(k).srate,[save_dir,'/',out(k).name,'.wav']);
                end
                
                
            end
            disp('...done');
            
            
        end
        
        
    end
    
    t=toc(tid);
    disp(['Total time taken: ',num2str(t),'sec(s)']);
end

end

